@@ -24,6 +24,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <string.h>
 
 #include "gdk-pixbuf-private.h"
@@ -598,11 +599,11 @@ gdk_pixbuf_loader_new_with_mime_type (const char *mime_type,
                 mimes = info->mime_types;
                 
                 for (j = 0; mimes[j] != NULL; j++)
-                        if (g_ascii_strcasecmp (mimes[i], mime_type)) {
+                        if (g_ascii_strcasecmp (mimes[j], mime_type) == 0) {
                                 image_type = info->name;
                                 break;
                         }
-        }
+        }        
 
         g_slist_free (formats);
 
