@@ -911,7 +911,7 @@ find_color(const char *name,
 	found = bsearch (name, xColors, numXColors, sizeof (XPMColorEntry),
 			 compare_xcolor_entries);
 	if (found == NULL)
-	  return 0;
+	  return FALSE;
 	
 	colorPtr->red = (found->red * 65535) / 255;
 	colorPtr->green = (found->green * 65535) / 255;
@@ -1079,7 +1079,7 @@ xpm_extract_color (const gchar *buffer)
 	gint key = 0;
 	gint current_key = 1;
 	gint space = 128;
-	gchar word[128], color[128], current_color[128];
+	gchar word[129], color[129], current_color[129];
 	gchar *r; 
 	
 	word[0] = '\0';
@@ -1121,8 +1121,8 @@ xpm_extract_color (const gchar *buffer)
 				return NULL;
 			/* accumulate color name */
 			if (color[0] != '\0') {
-				strcat (color, " ");
-				space--;
+				strncat (color, " ", space);
+				space -= MIN (space, 1);
 			}
 			strncat (color, word, space);
 			space -= MIN (space, strlen (word));
@@ -1208,7 +1208,7 @@ static GdkPixbuf *
 pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handle), gpointer handle,
                         GError **error)
 {
-	gint w, h, n_col, cpp;
+	gint w, h, n_col, cpp, x_hot, y_hot, items;
 	gint cnt, xcnt, ycnt, wbytes, n;
 	gint is_trans = FALSE;
 	const gchar *buffer;
@@ -1229,7 +1229,7 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
                              _("No XPM header found"));
 		return NULL;
 	}
-	sscanf (buffer, "%d %d %d %d", &w, &h, &n_col, &cpp);
+	items = sscanf (buffer, "%d %d %d %d %d %d", &w, &h, &n_col, &cpp, &x_hot, &y_hot);
 	if (w <= 0) {
                 g_set_error (error,
                              GDK_PIXBUF_ERROR,
@@ -1245,14 +1245,6 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
                              _("XPM file has image height <= 0"));
 		return NULL;
 
-	}
-	if (n_col <= 0) {
-                g_set_error (error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                             _("XPM file has invalid number of colors"));
-		return NULL;
-
 	}
 	if (cpp <= 0 || cpp >= 32) {
                 g_set_error (error,
@@ -1261,12 +1253,36 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
                              _("XPM has invalid number of chars per pixel"));
 		return NULL;
 	}
+	if (n_col <= 0 || n_col >= G_MAXINT / (cpp + 1)) {
+                g_set_error (error,
+                             GDK_PIXBUF_ERROR,
+                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                             _("XPM file has invalid number of colors"));
+		return NULL;
+	}
 
 	/* The hash is used for fast lookups of color from chars */
 	color_hash = g_hash_table_new (g_str_hash, g_str_equal);
 
-	name_buf = g_new (gchar, n_col * (cpp + 1));
-	colors = g_new (XPMColor, n_col);
+	name_buf = g_try_malloc (n_col * (cpp + 1));
+	if (!name_buf) {
+		g_set_error (error,
+			     GDK_PIXBUF_ERROR,
+                             GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                             _("Can't allocate memory for loading XPM image"));
+		g_hash_table_destroy (color_hash);
+		return NULL;
+	}
+	colors = (XPMColor *) g_try_malloc (sizeof (XPMColor) * n_col);
+	if (!colors) {
+		g_set_error (error,
+			     GDK_PIXBUF_ERROR,
+                             GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                             _("Can't allocate memory for loading XPM image"));
+		g_hash_table_destroy (color_hash);
+		g_free (name_buf);
+		return NULL;
+	}
 
 	for (cnt = 0; cnt < n_col; cnt++) {
 		gchar *color_name;
@@ -1355,6 +1371,15 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
 	g_free (colors);
 	g_free (name_buf);
 
+	if (items == 6) {
+		gchar hot[10];
+		g_snprintf (hot, 10, "%d", x_hot);
+		gdk_pixbuf_set_option (pixbuf, "x_hot", hot);
+		g_snprintf (hot, 10, "%d", y_hot);
+		gdk_pixbuf_set_option (pixbuf, "y_hot", hot);
+
+	}
+
 	return pixbuf;
 }
 
@@ -1400,8 +1425,8 @@ gdk_pixbuf__xpm_image_load_xpm_data (const gchar **data)
 typedef struct _XPMContext XPMContext;
 struct _XPMContext
 {
-       ModulePreparedNotifyFunc prepare_func;
-       ModuleUpdatedNotifyFunc update_func;
+       GdkPixbufModulePreparedFunc prepare_func;
+       GdkPixbufModuleUpdatedFunc update_func;
        gpointer user_data;
 
        gchar *tempname;
@@ -1416,8 +1441,9 @@ struct _XPMContext
  * in the future.
  */
 static gpointer
-gdk_pixbuf__xpm_image_begin_load (ModulePreparedNotifyFunc prepare_func,
-                                  ModuleUpdatedNotifyFunc update_func,
+gdk_pixbuf__xpm_image_begin_load (GdkPixbufModuleSizeFunc size_func,
+                                  GdkPixbufModulePreparedFunc prepare_func,
+                                  GdkPixbufModuleUpdatedFunc update_func,
                                   gpointer user_data,
                                   GError **error)
 {
@@ -1503,11 +1529,35 @@ gdk_pixbuf__xpm_image_load_increment (gpointer data,
 }
 
 void
-gdk_pixbuf__xpm_fill_vtable (GdkPixbufModule *module)
+MODULE_ENTRY (xpm, fill_vtable) (GdkPixbufModule *module)
 {
-  module->load = gdk_pixbuf__xpm_image_load;
-  module->load_xpm_data = gdk_pixbuf__xpm_image_load_xpm_data;
-  module->begin_load = gdk_pixbuf__xpm_image_begin_load;
-  module->stop_load = gdk_pixbuf__xpm_image_stop_load;
-  module->load_increment = gdk_pixbuf__xpm_image_load_increment;
+	module->load = gdk_pixbuf__xpm_image_load;
+	module->load_xpm_data = gdk_pixbuf__xpm_image_load_xpm_data;
+	module->begin_load = gdk_pixbuf__xpm_image_begin_load;
+	module->stop_load = gdk_pixbuf__xpm_image_stop_load;
+	module->load_increment = gdk_pixbuf__xpm_image_load_increment;
+}
+
+void
+MODULE_ENTRY (xpm, fill_info) (GdkPixbufFormat *info)
+{
+	static GdkPixbufModulePattern signature[] = {
+		{ "/* XPM */", NULL, 100 },
+		{ NULL, NULL, 0 }
+	};
+	static gchar * mime_types[] = {
+		"image/x-xpixmap",
+		NULL
+	};
+	static gchar * extensions[] = {
+		"xpm",
+		NULL
+	};
+
+	info->name = "xpm";
+	info->signature = signature;
+	info->description = N_("The XPM image format");
+	info->mime_types = mime_types;
+	info->extensions = extensions;
+	info->flags = 0;
 }
