@@ -24,6 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <string.h>
 
 #include "gdkgc.h"
@@ -57,7 +58,8 @@ gdk_gc_get_type (void)
       
       object_type = g_type_register_static (G_TYPE_OBJECT,
                                             "GdkGC",
-                                            &object_info, 0);
+                                            &object_info, 
+					    G_TYPE_FLAG_ABSTRACT);
     }
   
   return object_type;
@@ -111,7 +113,7 @@ gdk_gc_new_with_values (GdkDrawable	*drawable,
     {
       gc->colormap = gdk_drawable_get_colormap (drawable);
       if (gc->colormap)
-	g_object_ref (G_OBJECT (gc->colormap));
+	g_object_ref (gc->colormap);
     }
   
   return gc;
@@ -123,7 +125,7 @@ gdk_gc_finalize (GObject *object)
   GdkGC *gc = GDK_GC (object);
   
   if (gc->colormap)
-    g_object_unref (G_OBJECT (gc->colormap));
+    g_object_unref (gc->colormap);
 
   parent_class->finalize (object);
 }
@@ -139,7 +141,7 @@ gdk_gc_finalize (GObject *object)
 GdkGC *
 gdk_gc_ref (GdkGC *gc)
 {
-  return (GdkGC *) g_object_ref (G_OBJECT (gc));
+  return (GdkGC *) g_object_ref (gc);
 }
 
 /**
@@ -151,7 +153,7 @@ gdk_gc_ref (GdkGC *gc)
 void
 gdk_gc_unref (GdkGC *gc)
 {
-  g_object_unref (G_OBJECT (gc));
+  g_object_unref (gc);
 }
 
 void
@@ -198,8 +200,8 @@ gdk_gc_set_values (GdkGC           *gc,
 }
 
 void
-gdk_gc_set_foreground (GdkGC	*gc,
-		       GdkColor *color)
+gdk_gc_set_foreground (GdkGC	      *gc,
+		       const GdkColor *color)
 {
   GdkGCValues values;
 
@@ -211,8 +213,8 @@ gdk_gc_set_foreground (GdkGC	*gc,
 }
 
 void
-gdk_gc_set_background (GdkGC	*gc,
-		       GdkColor *color)
+gdk_gc_set_background (GdkGC	      *gc,
+		       const GdkColor *color)
 {
   GdkGCValues values;
 
@@ -438,10 +440,10 @@ gdk_gc_set_colormap (GdkGC       *gc,
   if (gc->colormap != colormap)
     {
       if (gc->colormap)
-	g_object_unref (G_OBJECT (gc->colormap));
+	g_object_unref (gc->colormap);
 
       gc->colormap = colormap;
-      g_object_ref (G_OBJECT (gc->colormap));
+      g_object_ref (gc->colormap);
     }
     
 }
@@ -495,7 +497,8 @@ gdk_gc_get_colormap_warn (GdkGC *gc)
  * Calling this function for a GC without a colormap is an error.
  **/
 void
-gdk_gc_set_rgb_fg_color (GdkGC *gc, GdkColor *color)
+gdk_gc_set_rgb_fg_color (GdkGC          *gc,
+			 const GdkColor *color)
 {
   GdkColormap *cmap;
   GdkColor tmp_color;
@@ -526,7 +529,8 @@ gdk_gc_set_rgb_fg_color (GdkGC *gc, GdkColor *color)
  * Calling this function for a GC without a colormap is an error.
  **/
 void
-gdk_gc_set_rgb_bg_color (GdkGC *gc, GdkColor *color)
+gdk_gc_set_rgb_bg_color (GdkGC          *gc,
+			 const GdkColor *color)
 {
   GdkColormap *cmap;
   GdkColor tmp_color;
