@@ -24,6 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <string.h>
@@ -514,7 +515,15 @@ make_list (const gchar  *text,
 	    }
 	}
       else
-	str = g_strndup (p, q - p);
+	{
+	  str = g_strndup (p, q - p);
+	  if (!g_utf8_validate (str, -1, NULL))
+	    {
+	      g_warning ("Error converting selection from UTF8_STRING");
+	      g_free (str);
+	      str = NULL;
+	    }
+	}
 
       if (str)
 	{
