@@ -24,6 +24,9 @@
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
+
+#include <config.h>
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,7 +37,6 @@
 #include "gtkimmodule.h"
 #include "gtkimcontextsimple.h"
 #include "gtkrc.h"
-#include "config.h"
 #include "gtkintl.h"
 
 /* Do *not* include "gtkprivate.h" in this file. If you do, the
@@ -517,10 +519,10 @@ match_locale (const gchar *locale,
   if (strcmp (against, "*") == 0)
     return 1;
 
-  if (strcmp (locale, against) == 0)
+  if (g_ascii_strcasecmp (locale, against) == 0)
     return 4;
 
-  if (strncmp (locale, against, 2) == 0)
+  if (g_ascii_strncasecmp (locale, against, 2) == 0)
     return (against_len == 2) ? 3 : 2;
 
   return 0;
