@@ -33,7 +33,7 @@
 #include "gdk/gdkkeysyms.h"
 #include "x11/gdkx.h"
 
-#include "xembed.h"
+#include "gtkxembed.h"
 
 static void            gtk_plug_class_init            (GtkPlugClass     *klass);
 static void            gtk_plug_init                  (GtkPlug          *plug);
@@ -61,12 +61,6 @@ static GdkFilterReturn gtk_plug_filter_func           (GdkXEvent        *gdk_xev
 						       gpointer          data);
 
 static void handle_modality_off        (GtkPlug       *plug);
-static void send_xembed_message        (GtkPlug       *plug,
-					glong          message,
-					glong          detail,
-					glong          data1,
-					glong          data2,
-					guint32        time);
 static void xembed_set_info            (GdkWindow     *window,
 					unsigned long  flags);
 
@@ -83,10 +77,10 @@ enum {
 
 static guint plug_signals[LAST_SIGNAL] = { 0 };
 
-GtkType
+GType
 gtk_plug_get_type ()
 {
-  static GtkType plug_type = 0;
+  static GType plug_type = 0;
 
   if (!plug_type)
     {
@@ -103,7 +97,8 @@ gtk_plug_get_type ()
 	(GInstanceInitFunc) gtk_plug_init,
       };
 
-      plug_type = g_type_register_static (GTK_TYPE_WINDOW, "GtkPlug", &plug_info, 0);
+      plug_type = g_type_register_static (GTK_TYPE_WINDOW, "GtkPlug",
+					  &plug_info, 0);
     }
 
   return plug_type;
@@ -117,8 +112,8 @@ gtk_plug_class_init (GtkPlugClass *class)
   GtkWindowClass *window_class = (GtkWindowClass *)class;
   GtkContainerClass *container_class = (GtkContainerClass *)class;
 
-  parent_class = gtk_type_class (GTK_TYPE_WINDOW);
-  bin_class = gtk_type_class (GTK_TYPE_BIN);
+  parent_class = g_type_class_peek_parent (class);
+  bin_class = g_type_class_peek (GTK_TYPE_BIN);
 
   gobject_class->finalize = gtk_plug_finalize;
   
@@ -148,7 +143,7 @@ gtk_plug_class_init (GtkPlugClass *class)
 		  G_STRUCT_OFFSET (GtkPlugClass, embedded),
 		  NULL, NULL,
 		  _gtk_marshal_VOID__VOID,
-		  GTK_TYPE_NONE, 0);
+		  G_TYPE_NONE, 0);
 }
 
 static void
@@ -212,7 +207,7 @@ gtk_plug_set_is_child (GtkPlug  *plug,
 /**
  * _gtk_plug_add_to_socket:
  * @plug: a #GtkPlug
- * @socket: a #GtkSocket
+ * @socket_: a #GtkSocket
  * 
  * Adds a plug to a socket within the same application.
  **/
@@ -244,13 +239,31 @@ _gtk_plug_add_to_socket (GtkPlug   *plug,
 
   gtk_widget_set_parent (widget, GTK_WIDGET (socket));
 
-  g_signal_emit_by_name (G_OBJECT (socket), "plug_added", 0);
+  g_signal_emit_by_name (socket, "plug_added", 0);
+}
+
+static void
+send_delete_event (GtkWidget *widget)
+{
+  GdkEvent *event = gdk_event_new (GDK_DELETE);
+  
+  event->any.window = g_object_ref (widget->window);
+  event->any.send_event = FALSE;
+
+  gtk_widget_ref (widget);
+  
+  if (!gtk_widget_event (widget, event))
+    gtk_widget_destroy (widget);
+  
+  gtk_widget_unref (widget);
+
+  gdk_event_free (event);
 }
 
 /**
  * _gtk_plug_remove_from_socket:
  * @plug: a #GtkPlug
- * @socket: a #GtkSocket
+ * @socket_: a #GtkSocket
  * 
  * Removes a plug from a socket within the same application.
  **/
@@ -259,7 +272,6 @@ _gtk_plug_remove_from_socket (GtkPlug   *plug,
 			      GtkSocket *socket)
 {
   GtkWidget *widget;
-  GdkEvent event;
   gboolean result;
   gboolean widget_was_visible;
 
@@ -284,8 +296,11 @@ _gtk_plug_remove_from_socket (GtkPlug   *plug,
   GTK_PRIVATE_UNSET_FLAG (plug, GTK_IN_REPARENT);
   
   socket->plug_widget = NULL;
-  g_object_unref (socket->plug_window);
-  socket->plug_window = NULL;
+  if (socket->plug_window != NULL)
+    {
+      g_object_unref (socket->plug_window);
+      socket->plug_window = NULL;
+    }
   
   socket->same_app = FALSE;
 
@@ -294,18 +309,12 @@ _gtk_plug_remove_from_socket (GtkPlug   *plug,
 
   gtk_plug_set_is_child (plug, FALSE);
 		    
-  g_signal_emit_by_name (G_OBJECT (socket), "plug_removed", &result);
+  g_signal_emit_by_name (socket, "plug_removed", &result);
   if (!result)
     gtk_widget_destroy (GTK_WIDGET (socket));
 
-  event.any.type = GDK_DELETE;
-  event.any.window = g_object_ref (widget->window);
-  event.any.send_event = FALSE;
-  
-  if (!gtk_widget_event (widget, &event))
-    gtk_widget_destroy (widget);
-  
-  g_object_unref (event.any.window);
+  send_delete_event (widget);
+
   g_object_unref (plug);
 
   if (widget_was_visible && GTK_WIDGET_VISIBLE (socket))
@@ -339,6 +348,8 @@ gtk_plug_construct (GtkPlug         *plug,
  * Finish the initialization of @plug for a given #GtkSocket identified by
  * @socket_id which is currently displayed on @display.
  * This function will generally only be used by classes deriving from #GtkPlug.
+ *
+ * Since: 2.2
  **/
 void
 gtk_plug_construct_for_display (GtkPlug         *plug,
@@ -368,7 +379,7 @@ gtk_plug_construct_for_display (GtkPlug         *plug,
 	}
 
       if (plug->socket_window)
-	g_signal_emit (G_OBJECT (plug), plug_signals[EMBEDDED], 0);
+	g_signal_emit (plug, plug_signals[EMBEDDED], 0);
     }
 }
 
@@ -396,6 +407,8 @@ gtk_plug_new (GdkNativeWindow socket_id)
  * Create a new plug widget inside the #GtkSocket identified by socket_id.
  *
  * Return value: the new #GtkPlug widget.
+ *
+ * Since: 2.2
  */
 GtkWidget*
 gtk_plug_new_for_display (GdkDisplay	  *display,
@@ -403,7 +416,7 @@ gtk_plug_new_for_display (GdkDisplay	  *display,
 {
   GtkPlug *plug;
 
-  plug = GTK_PLUG (gtk_type_new (GTK_TYPE_PLUG));
+  plug = g_object_new (GTK_TYPE_PLUG, NULL);
   gtk_plug_construct_for_display (plug, display, socket_id);
   return GTK_WIDGET (plug);
 }
@@ -455,7 +468,7 @@ gtk_plug_unrealize (GtkWidget *widget)
   if (plug->socket_window != NULL)
     {
       gdk_window_set_user_data (plug->socket_window, NULL);
-      gdk_window_unref (plug->socket_window);
+      g_object_unref (plug->socket_window);
       plug->socket_window = NULL;
     }
 
@@ -682,8 +695,8 @@ gtk_plug_set_focus (GtkWindow *window,
 
   if (focus && !window->has_toplevel_focus)
     {
-      send_xembed_message (plug, XEMBED_REQUEST_FOCUS, 0, 0, 0,
-			   gtk_get_current_event_time ());
+      _gtk_xembed_send_message (plug->socket_window,
+				XEMBED_REQUEST_FOCUS, 0, 0, 0);
     }
 }
 
@@ -725,9 +738,8 @@ add_grabbed_key (gpointer key, gpointer val, gpointer data)
   if (!plug->grabbed_keys ||
       !g_hash_table_lookup (plug->grabbed_keys, grabbed_key))
     {
-      send_xembed_message (plug, XEMBED_GTK_GRAB_KEY, 0, 
-			   grabbed_key->accelerator_key, grabbed_key->accelerator_mods,
-			   gtk_get_current_event_time ());
+      _gtk_xembed_send_message (plug->socket_window, XEMBED_GTK_GRAB_KEY, 0, 
+				grabbed_key->accelerator_key, grabbed_key->accelerator_mods);
     }
 }
 
@@ -737,9 +749,8 @@ add_grabbed_key_always (gpointer key, gpointer val, gpointer data)
   GrabbedKey *grabbed_key = key;
   GtkPlug *plug = data;
 
-  send_xembed_message (plug, XEMBED_GTK_GRAB_KEY, 0, 
-		       grabbed_key->accelerator_key, grabbed_key->accelerator_mods,
-		       gtk_get_current_event_time ());
+  _gtk_xembed_send_message (plug->socket_window, XEMBED_GTK_GRAB_KEY, 0, 
+			    grabbed_key->accelerator_key, grabbed_key->accelerator_mods);
 }
 
 static void
@@ -751,9 +762,8 @@ remove_grabbed_key (gpointer key, gpointer val, gpointer data)
   if (!plug->grabbed_keys ||
       !g_hash_table_lookup (plug->grabbed_keys, grabbed_key))
     {
-      send_xembed_message (plug, XEMBED_GTK_UNGRAB_KEY, 0, 
-			   grabbed_key->accelerator_key, grabbed_key->accelerator_mods,
-			   gtk_get_current_event_time ());
+      _gtk_xembed_send_message (plug->socket_window, XEMBED_GTK_UNGRAB_KEY, 0, 
+				grabbed_key->accelerator_key, grabbed_key->accelerator_mods);
     }
 }
 
@@ -796,6 +806,29 @@ gtk_plug_keys_changed (GtkWindow *window)
     }
 }
 
+static void
+focus_to_parent (GtkPlug          *plug,
+		 GtkDirectionType  direction)
+{
+  XEmbedMessageType message = XEMBED_FOCUS_PREV; /* Quiet GCC */
+  
+  switch (direction)
+    {
+    case GTK_DIR_UP:
+    case GTK_DIR_LEFT:
+    case GTK_DIR_TAB_BACKWARD:
+      message = XEMBED_FOCUS_PREV;
+      break;
+    case GTK_DIR_DOWN:
+    case GTK_DIR_RIGHT:
+    case GTK_DIR_TAB_FORWARD:
+      message = XEMBED_FOCUS_NEXT;
+      break;
+    }
+  
+  _gtk_xembed_send_focus_message (plug->socket_window, message, 0);
+}
+
 static gboolean
 gtk_plug_focus (GtkWidget        *widget,
 		GtkDirectionType  direction)
@@ -825,40 +858,18 @@ gtk_plug_focus (GtkWidget        *widget,
 	    }
 	  
 	  gtk_window_set_focus (GTK_WINDOW (container), NULL);
-
-	  if (!GTK_CONTAINER (window)->focus_child)
-	    {
-	      gint message = -1;
-
-	      switch (direction)
-		{
-		case GTK_DIR_UP:
-		case GTK_DIR_LEFT:
-		case GTK_DIR_TAB_BACKWARD:
-		  message = XEMBED_FOCUS_PREV;
-		  break;
-		case GTK_DIR_DOWN:
-		case GTK_DIR_RIGHT:
-		case GTK_DIR_TAB_FORWARD:
-		  message = XEMBED_FOCUS_NEXT;
-		  break;
-		}
-	      
-	      send_xembed_message (plug, message, 0, 0, 0,
-				   gtk_get_current_event_time ());
-	    }
 	}
-
-      return FALSE;
     }
   else
     {
       /* Try to focus the first widget in the window */
-      
       if (bin->child && gtk_widget_child_focus (bin->child, direction))
         return TRUE;
     }
 
+  if (!GTK_CONTAINER (window)->focus_child)
+    focus_to_parent (plug, direction);
+
   return FALSE;
 }
 
@@ -871,48 +882,13 @@ gtk_plug_check_resize (GtkContainer *container)
     GTK_CONTAINER_CLASS (bin_class)->check_resize (container);
 }
 
-static void
-send_xembed_message (GtkPlug *plug,
-		     glong      message,
-		     glong      detail,
-		     glong      data1,
-		     glong      data2,
-		     guint32    time)
-{
-  if (plug->socket_window)
-    {
-      GdkDisplay *display = gdk_drawable_get_display (plug->socket_window);
-      XEvent xevent;
-
-      GTK_NOTE(PLUGSOCKET,
-	       g_message ("GtkPlug: Sending XEMBED message of type %ld", message));
-
-      xevent.xclient.window = GDK_WINDOW_XWINDOW (plug->socket_window);
-      xevent.xclient.type = ClientMessage;
-      xevent.xclient.message_type = gdk_x11_get_xatom_by_name_for_display (display, "_XEMBED");
-      xevent.xclient.format = 32;
-      xevent.xclient.data.l[0] = time;
-      xevent.xclient.data.l[1] = message;
-      xevent.xclient.data.l[2] = detail;
-      xevent.xclient.data.l[3] = data1;
-      xevent.xclient.data.l[4] = data2;
-
-      gdk_error_trap_push ();
-      XSendEvent (GDK_WINDOW_XDISPLAY(plug->socket_window),
-		  GDK_WINDOW_XWINDOW (plug->socket_window),
-		  False, NoEventMask, &xevent);
-      gdk_display_sync (display);
-      gdk_error_trap_pop ();
-    }
-}
-
 static void
 focus_first_last (GtkPlug          *plug,
 		  GtkDirectionType  direction)
 {
   GtkWindow *window = GTK_WINDOW (plug);
   GtkWidget *parent;
-  
+
   if (window->focus_widget)
     {
       parent = window->focus_widget->parent;
@@ -961,7 +937,7 @@ xembed_set_info (GdkWindow     *window,
 
   Atom xembed_info_atom = gdk_x11_get_xatom_by_name_for_display (display, "_XEMBED_INFO");
 
-  buffer[1] = 0;		/* Protocol version */
+  buffer[0] = GTK_XEMBED_PROTOCOL_VERSION;
   buffer[1] = flags;
 
   XChangeProperty (GDK_DISPLAY_XDISPLAY (display),
@@ -972,17 +948,17 @@ xembed_set_info (GdkWindow     *window,
 }
 
 static void
-handle_xembed_message (GtkPlug   *plug,
-		       glong      message,
-		       glong      detail,
-		       glong      data1,
-		       glong      data2,
-		       guint32    time)
+handle_xembed_message (GtkPlug           *plug,
+		       XEmbedMessageType  message,
+		       glong              detail,
+		       glong              data1,
+		       glong              data2,
+		       guint32            time)
 {
   GtkWindow *window = GTK_WINDOW (plug);
-  
+
   GTK_NOTE (PLUGSOCKET,
-	    g_message ("GtkPlug: Message of type %ld received", message));
+	    g_message ("GtkPlug: Message of type %d received", message));
   
   switch (message)
     {
@@ -1028,12 +1004,12 @@ handle_xembed_message (GtkPlug   *plug,
     case XEMBED_REQUEST_FOCUS:
     case XEMBED_FOCUS_NEXT:
     case XEMBED_FOCUS_PREV:
-      g_warning ("GtkPlug: Invalid _XEMBED message of type %ld received", message);
+      g_warning ("GtkPlug: Invalid _XEMBED message of type %d received", message);
       break;
       
     default:
       GTK_NOTE(PLUGSOCKET,
-	       g_message ("GtkPlug: Ignoring unknown _XEMBED message of type %ld", message));
+	       g_message ("GtkPlug: Ignoring unknown _XEMBED message of type %d", message));
       break;
     }
 }
@@ -1055,14 +1031,15 @@ gtk_plug_filter_func (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
     case ClientMessage:
       if (xevent->xclient.message_type == gdk_x11_get_xatom_by_name_for_display (display, "_XEMBED"))
 	{
+	  _gtk_xembed_push_message (xevent);
 	  handle_xembed_message (plug,
 				 xevent->xclient.data.l[1],
 				 xevent->xclient.data.l[2],
 				 xevent->xclient.data.l[3],
 				 xevent->xclient.data.l[4],
 				 xevent->xclient.data.l[0]);
+	  _gtk_xembed_pop_message ();
 				 
-
 	  return GDK_FILTER_REMOVE;
 	}
       else if (xevent->xclient.message_type == gdk_x11_get_xatom_by_name_for_display (display, "WM_DELETE_WINDOW"))
@@ -1097,7 +1074,7 @@ gtk_plug_filter_func (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 		GtkWidget *widget = GTK_WIDGET (plug);
 
 		gdk_window_set_user_data (plug->socket_window, NULL);
-		gdk_window_unref (plug->socket_window);
+		g_object_unref (plug->socket_window);
 		plug->socket_window = NULL;
 
 		/* Emit a delete window, as if the user attempted
@@ -1110,18 +1087,7 @@ gtk_plug_filter_func (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 		 */
 
 		if (xre->parent == GDK_WINDOW_XWINDOW (gdk_screen_get_root_window (screen)))
-		  {
-		    GdkEvent event;
-		    
-		    event.any.type = GDK_DELETE;
-		    event.any.window = g_object_ref (widget->window);
-		    event.any.send_event = FALSE;
-		    
-		    if (!gtk_widget_event (widget, &event))
-		      gtk_widget_destroy (widget);
-		    
-		    g_object_unref (event.any.window);
-		  }
+		  send_delete_event (widget);
 	      }
 	    else
 	      goto done;
@@ -1157,7 +1123,7 @@ gtk_plug_filter_func (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 	      g_hash_table_foreach (plug->grabbed_keys, add_grabbed_key_always, plug);
 
 	    if (!was_embedded)
-	      g_signal_emit (G_OBJECT (plug), plug_signals[EMBEDDED], 0);
+	      g_signal_emit (plug, plug_signals[EMBEDDED], 0);
 	  }
 
       done:
