@@ -163,7 +163,12 @@ struct _GtkTextLayout
   /* For what GtkTextDirection to draw cursor GTK_TEXT_DIR_NONE -
    * means draw both cursors.
    */
-  gint cursor_direction : 2;
+  guint cursor_direction : 2;
+
+  /* The keyboard direction is used to default the alignment when
+     there are no strong characters.
+  */
+  guint keyboard_direction : 2;
 
   /* The preedit string and attributes, if any */
 
@@ -206,8 +211,11 @@ struct _GtkTextLayoutClass
                                  gint               x,
                                  gint               y);
 
-  GtkFunction pad1;
-  GtkFunction pad2;
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 struct _GtkTextAttrAppearance
@@ -262,7 +270,9 @@ void               gtk_text_layout_set_contexts          (GtkTextLayout     *lay
 							  PangoContext      *ltr_context,
 							  PangoContext      *rtl_context);
 void               gtk_text_layout_set_cursor_direction  (GtkTextLayout     *layout,
-							  GtkTextDirection   direction);
+                                                          GtkTextDirection   direction);
+void               gtk_text_layout_set_keyboard_direction (GtkTextLayout     *layout,
+							   GtkTextDirection keyboard_dir);
 void               gtk_text_layout_default_style_changed (GtkTextLayout     *layout);
 
 void gtk_text_layout_set_screen_width       (GtkTextLayout     *layout,
@@ -319,8 +329,8 @@ void gtk_text_layout_free_line_data    (GtkTextLayout     *layout,
 gboolean gtk_text_layout_is_valid        (GtkTextLayout *layout);
 void     gtk_text_layout_validate_yrange (GtkTextLayout *layout,
                                           GtkTextIter   *anchor_line,
-                                          gint           y0,
-                                          gint           y1);
+                                          gint           y0_,
+                                          gint           y1_);
 void     gtk_text_layout_validate        (GtkTextLayout *layout,
                                           gint           max_pixels);
 
@@ -345,6 +355,10 @@ void     gtk_text_layout_get_line_yrange      (GtkTextLayout     *layout,
                                                const GtkTextIter *iter,
                                                gint              *y,
                                                gint              *height);
+void     _gtk_text_layout_get_line_xrange     (GtkTextLayout     *layout,
+                                               const GtkTextIter *iter,
+                                               gint              *x,
+                                               gint              *width);
 void     gtk_text_layout_get_cursor_locations (GtkTextLayout     *layout,
                                                GtkTextIter       *iter,
                                                GdkRectangle      *strong_pos,
