@@ -24,10 +24,10 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gtklabel.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
-#include "gtksignal.h"
 #include "gtktogglebutton.h"
 #include "gtkintl.h"
 
@@ -50,10 +50,10 @@ enum {
 
 static void gtk_toggle_button_class_init    (GtkToggleButtonClass *klass);
 static void gtk_toggle_button_init          (GtkToggleButton      *toggle_button);
-static void gtk_toggle_button_paint         (GtkWidget            *widget,
-					     GdkRectangle         *area);
 static gint gtk_toggle_button_expose        (GtkWidget            *widget,
 					     GdkEventExpose       *event);
+static gboolean gtk_toggle_button_mnemonic_activate  (GtkWidget            *widget,
+                                                      gboolean              group_cycling);
 static void gtk_toggle_button_pressed       (GtkButton            *button);
 static void gtk_toggle_button_released      (GtkButton            *button);
 static void gtk_toggle_button_clicked       (GtkButton            *button);
@@ -70,26 +70,29 @@ static void gtk_toggle_button_update_state  (GtkButton            *button);
 static guint toggle_button_signals[LAST_SIGNAL] = { 0 };
 static GtkContainerClass *parent_class = NULL;
 
-GtkType
+GType
 gtk_toggle_button_get_type (void)
 {
-  static GtkType toggle_button_type = 0;
+  static GType toggle_button_type = 0;
 
   if (!toggle_button_type)
     {
-      static const GtkTypeInfo toggle_button_info =
+      static const GTypeInfo toggle_button_info =
       {
-	"GtkToggleButton",
-	sizeof (GtkToggleButton),
 	sizeof (GtkToggleButtonClass),
-	(GtkClassInitFunc) gtk_toggle_button_class_init,
-	(GtkObjectInitFunc) gtk_toggle_button_init,
-        /* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-	(GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_toggle_button_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkToggleButton),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_toggle_button_init,
       };
 
-      toggle_button_type = gtk_type_unique (GTK_TYPE_BUTTON, &toggle_button_info);
+      toggle_button_type =
+	g_type_register_static (GTK_TYPE_BUTTON, "GtkToggleButton",
+				&toggle_button_info, 0);
     }
 
   return toggle_button_type;
@@ -98,25 +101,23 @@ gtk_toggle_button_get_type (void)
 static void
 gtk_toggle_button_class_init (GtkToggleButtonClass *class)
 {
-  GtkObjectClass *object_class;
-  GObjectClass   *gobject_class;
+  GObjectClass *gobject_class;
   GtkWidgetClass *widget_class;
   GtkContainerClass *container_class;
   GtkButtonClass *button_class;
 
-  object_class = (GtkObjectClass*) class;
   gobject_class = G_OBJECT_CLASS (class);
   widget_class = (GtkWidgetClass*) class;
   container_class = (GtkContainerClass*) class;
   button_class = (GtkButtonClass*) class;
 
-  parent_class = gtk_type_class (GTK_TYPE_BUTTON);
-
+  parent_class = g_type_class_peek_parent (class);
 
   gobject_class->set_property = gtk_toggle_button_set_property;
   gobject_class->get_property = gtk_toggle_button_get_property;
 
   widget_class->expose_event = gtk_toggle_button_expose;
+  widget_class->mnemonic_activate = gtk_toggle_button_mnemonic_activate;
 
   button_class->pressed = gtk_toggle_button_pressed;
   button_class->released = gtk_toggle_button_released;
@@ -129,34 +130,35 @@ gtk_toggle_button_class_init (GtkToggleButtonClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVE,
                                    g_param_spec_boolean ("active",
-							 _("Active"),
-							 _("If the toggle button should be pressed in or not"),
+							 P_("Active"),
+							 P_("If the toggle button should be pressed in or not"),
 							 FALSE,
 							 G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_INCONSISTENT,
                                    g_param_spec_boolean ("inconsistent",
-							 _("Inconsistent"),
-							 _("If the toggle button is in an \"in between\" state."),
+							 P_("Inconsistent"),
+							 P_("If the toggle button is in an \"in between\" state"),
 							 FALSE,
 							 G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_DRAW_INDICATOR,
                                    g_param_spec_boolean ("draw_indicator",
-							 _("Draw Indicator"),
-							 _("If the toggle part of the button is displayed"),
+							 P_("Draw Indicator"),
+							 P_("If the toggle part of the button is displayed"),
 							 FALSE,
 							 G_PARAM_READWRITE));
 
   toggle_button_signals[TOGGLED] =
-    gtk_signal_new ("toggled",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkToggleButtonClass, toggled),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("toggled",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkToggleButtonClass, toggled),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 }
 
 static void
@@ -164,13 +166,14 @@ gtk_toggle_button_init (GtkToggleButton *toggle_button)
 {
   toggle_button->active = FALSE;
   toggle_button->draw_indicator = FALSE;
+  GTK_BUTTON (toggle_button)->depress_on_activate = TRUE;
 }
 
 
 GtkWidget*
 gtk_toggle_button_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (gtk_toggle_button_get_type ()));
+  return g_object_new (GTK_TYPE_TOGGLE_BUTTON, NULL);
 }
 
 GtkWidget*
@@ -248,6 +251,20 @@ gtk_toggle_button_get_property (GObject      *object,
     }
 }
 
+/**
+ * gtk_toggle_button_set_mode:
+ * @toggle_button: a #GtkToggleButton
+ * @draw_indicator: if %TRUE, draw the button as a separate indicator
+ * and label; if %FALSE, draw the button like a normal button
+ *
+ * Sets whether the button is displayed as a separate indicator and label.
+ * You can call this function on a checkbutton or a radiobutton with
+ * @draw_indicator = %FALSE to make the button look like a normal button
+ *
+ * This function only effects instances of classes like #GtkCheckButton
+ * and #GtkRadioButton that derive from #GtkToggleButton,
+ * not instances of #GtkToggleButton itself.
+ */
 void
 gtk_toggle_button_set_mode (GtkToggleButton *toggle_button,
 			    gboolean         draw_indicator)
@@ -263,7 +280,8 @@ gtk_toggle_button_set_mode (GtkToggleButton *toggle_button,
   if (toggle_button->draw_indicator != draw_indicator)
     {
       toggle_button->draw_indicator = draw_indicator;
-
+      GTK_BUTTON (toggle_button)->depress_on_activate = !draw_indicator;
+      
       if (GTK_WIDGET_VISIBLE (toggle_button))
 	gtk_widget_queue_resize (GTK_WIDGET (toggle_button));
 
@@ -316,7 +334,7 @@ gtk_toggle_button_toggled (GtkToggleButton *toggle_button)
 {
   g_return_if_fail (GTK_IS_TOGGLE_BUTTON (toggle_button));
 
-  gtk_signal_emit (GTK_OBJECT (toggle_button), toggle_button_signals[TOGGLED]);
+  g_signal_emit (toggle_button, toggle_button_signals[TOGGLED], 0);
 }
 
 /**
@@ -369,63 +387,20 @@ gtk_toggle_button_get_inconsistent (GtkToggleButton *toggle_button)
   return toggle_button->inconsistent;
 }
 
-static void
-gtk_toggle_button_paint (GtkWidget    *widget,
-			 GdkRectangle *area)
+static gint
+gtk_toggle_button_expose (GtkWidget      *widget,
+			  GdkEventExpose *event)
 {
-  GtkButton *button;
-  GtkToggleButton *toggle_button;
-  GtkShadowType shadow_type;
-  GtkStateType state_type;
-  gint width, height;
-  gboolean interior_focus;
-  gint border_width;
-  gint x, y;
-
-  button = GTK_BUTTON (widget);
-  toggle_button = GTK_TOGGLE_BUTTON (widget);
-
   if (GTK_WIDGET_DRAWABLE (widget))
     {
-      border_width = GTK_CONTAINER (widget)->border_width;
-      
-      gtk_widget_style_get (widget, "interior_focus", &interior_focus, NULL);
-      
-      x = widget->allocation.x + border_width;
-      y = widget->allocation.y + border_width;
-      width = widget->allocation.width - border_width * 2;
-      height = widget->allocation.height - border_width * 2;
-
-      if (GTK_WIDGET_HAS_DEFAULT (widget) &&
-          GTK_BUTTON (widget)->relief == GTK_RELIEF_NORMAL)
-        {
-          gtk_paint_box (widget->style, widget->window,
-                         GTK_STATE_NORMAL, GTK_SHADOW_IN,
-                         area, widget, "togglebuttondefault",
-                         x, y, width, height);
-        }
-
-      if (GTK_WIDGET_CAN_DEFAULT (widget))
-        {
-          x += widget->style->xthickness;
-          y += widget->style->ythickness;
-          width -= 2 * x + DEFAULT_SPACING;
-          height -= 2 * y + DEFAULT_SPACING;
-          x += DEFAULT_LEFT_POS;
-          y += DEFAULT_TOP_POS;
-        }
-
-      if (GTK_WIDGET_HAS_FOCUS (widget) && !interior_focus)
-	{
-	  x += 1;
-	  y += 1;
-	  width -= 2;
-	  height -= 2;
-	}
+      GtkWidget *child = GTK_BIN (widget)->child;
+      GtkButton *button = GTK_BUTTON (widget);
+      GtkStateType state_type;
+      GtkShadowType shadow_type;
 
       state_type = GTK_WIDGET_STATE (widget);
       
-      if (toggle_button->inconsistent)
+      if (GTK_TOGGLE_BUTTON (widget)->inconsistent)
         {
           if (state_type == GTK_STATE_ACTIVE)
             state_type = GTK_STATE_NORMAL;
@@ -434,52 +409,31 @@ gtk_toggle_button_paint (GtkWidget    *widget,
       else
 	shadow_type = button->depressed ? GTK_SHADOW_IN : GTK_SHADOW_OUT;
 
-      if (button->relief != GTK_RELIEF_NONE ||
-	  (GTK_WIDGET_STATE(widget) != GTK_STATE_NORMAL &&
-	   GTK_WIDGET_STATE(widget) != GTK_STATE_INSENSITIVE))
-	gtk_paint_box (widget->style, widget->window,
-                       state_type,
-		       shadow_type, area, widget, "togglebutton",
-		       x, y, width, height);
-      
-      if (GTK_WIDGET_HAS_FOCUS (widget))
-	{
-	  if (interior_focus)
-	    {
-	      x += widget->style->xthickness + 1;
-	      y += widget->style->xthickness + 1;
-	      width -= 2 * (widget->style->xthickness + 1);
-	      height -= 2 * (widget->style->ythickness + 1);
-	    }
-	  else
-	    {
-	      x -= 1;
-	      y -= 1;
-	      width += 2;
-	      height += 2;
-	    }
-
-	  gtk_paint_focus (widget->style, widget->window,
-			   area, widget, "togglebutton",
-			   x, y, width - 1, height - 1);
-	}
-    }
-}
-
-static gint
-gtk_toggle_button_expose (GtkWidget      *widget,
-			  GdkEventExpose *event)
-{
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      GtkWidget *child = GTK_BIN (widget)->child;
-
-      gtk_toggle_button_paint (widget, &event->area);
+      _gtk_button_paint (button, &event->area, state_type, shadow_type,
+			 "togglebutton", "togglebuttondefault");
 
       if (child)
 	gtk_container_propagate_expose (GTK_CONTAINER (widget), child, event);
     }
   
+  return FALSE;
+}
+
+static gboolean
+gtk_toggle_button_mnemonic_activate (GtkWidget *widget,
+                                     gboolean   group_cycling)
+{
+  /*
+   * We override the standard implementation in 
+   * gtk_widget_real_mnemonic_activate() in order to focus the widget even
+   * if there is no mnemonic conflict.
+   */
+  if (GTK_WIDGET_CAN_FOCUS (widget))
+    gtk_widget_grab_focus (widget);
+
+  if (!group_cycling)
+    gtk_widget_activate (widget);
+
   return TRUE;
 }
 
@@ -489,6 +443,7 @@ gtk_toggle_button_pressed (GtkButton *button)
   button->button_down = TRUE;
 
   gtk_toggle_button_update_state (button);
+  gtk_widget_queue_draw (GTK_WIDGET (button));
 }
 
 static void
@@ -502,6 +457,7 @@ gtk_toggle_button_released (GtkButton *button)
 	gtk_button_clicked (button);
 
       gtk_toggle_button_update_state (button);
+      gtk_widget_queue_draw (GTK_WIDGET (button));
     }
 }
 
@@ -528,14 +484,14 @@ gtk_toggle_button_update_state (GtkButton *button)
   if (toggle_button->inconsistent)
     depressed = FALSE;
   else if (button->in_button && button->button_down)
-    depressed = !toggle_button->active;
+    depressed = TRUE;
   else
     depressed = toggle_button->active;
       
-  if (!button->button_down && button->in_button)
+  if (button->in_button && (!button->button_down || toggle_button->draw_indicator))
     new_state = GTK_STATE_PRELIGHT;
   else
-    new_state = depressed ? GTK_STATE_ACTIVE: GTK_STATE_NORMAL;
+    new_state = depressed ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL;
 
   _gtk_button_set_depressed (button, depressed); 
   gtk_widget_set_state (GTK_WIDGET (toggle_button), new_state);
